@@ -1,3 +1,4 @@
+/* SPDX-License-Identifier: MIT OR GPL-3.0-only */
 /* event.c
 ** strophe XMPP client library -- event loop and management
 **
@@ -6,7 +7,7 @@
 **  This software is provided AS-IS with no warranty, either express
 **  or implied.
 **
-** This program is dual licensed under the MIT and GPLv3 licenses.
+** This program is dual licensed under the MIT or GPLv3 licenses.
 */
 
 /** @file
@@ -57,8 +58,10 @@
 #include "common.h"
 #include "parser.h"
 
+#ifndef STROPHE_MESSAGE_BUFFER_SIZE
 /** Max buffer size for receiving messages. */
-#define STROPE_MESSAGE_BUFFER_SIZE 4096
+#define STROPHE_MESSAGE_BUFFER_SIZE 4096
+#endif
 
 static int _connect_next(xmpp_conn_t *conn)
 {
@@ -89,13 +92,14 @@ void xmpp_run_once(xmpp_ctx_t *ctx, unsigned long timeout)
 {
     xmpp_connlist_t *connitem;
     xmpp_conn_t *conn;
+    struct conn_interface *intf;
     fd_set rfds, wfds;
     sock_t max = 0;
     int ret;
     struct timeval tv;
     xmpp_send_queue_t *sq, *tsq;
     int towrite;
-    char buf[STROPE_MESSAGE_BUFFER_SIZE];
+    char buf[STROPHE_MESSAGE_BUFFER_SIZE];
     uint64_t next;
     uint64_t usec;
     int tls_read_bytes = 0;
@@ -111,16 +115,18 @@ void xmpp_run_once(xmpp_ctx_t *ctx, unsigned long timeout)
             connitem = connitem->next;
             continue;
         }
+        intf = &conn->intf;
 
         /* if we're running tls, there may be some remaining data waiting to
          * be sent, so push that out */
         if (conn->tls) {
-            ret = tls_clear_pending_write(conn->tls);
+            ret = tls_clear_pending_write(intf);
 
-            if (ret < 0 && !tls_is_recoverable(tls_error(conn->tls))) {
+            if (ret < 0 && !tls_is_recoverable(intf, tls_error(intf))) {
                 /* an error occurred */
-                strophe_debug(ctx, "xmpp",
-                              "Send error occurred, disconnecting.");
+                strophe_debug(
+                    ctx, "xmpp",
+                    "Send error of pending data occurred, disconnecting.");
                 conn->error = ECONNABORTED;
                 conn_disconnect(conn);
                 goto next_item;
@@ -132,15 +138,7 @@ void xmpp_run_once(xmpp_ctx_t *ctx, unsigned long timeout)
         while (sq) {
             towrite = sq->len - sq->written;
 
-            if (conn->tls) {
-                ret = tls_write(conn->tls, &sq->data[sq->written], towrite);
-                if (ret < 0 && !tls_is_recoverable(tls_error(conn->tls)))
-                    conn->error = tls_error(conn->tls);
-            } else {
-                ret = sock_write(conn->sock, &sq->data[sq->written], towrite);
-                if (ret < 0 && !sock_is_recoverable(sock_error()))
-                    conn->error = sock_error();
-            }
+            ret = conn_interface_write(intf, &sq->data[sq->written], towrite);
             if (ret > 0 && ret < towrite)
                 sq->written += ret; /* not all data could be sent now */
             sq->wip = 1;
@@ -158,7 +156,8 @@ void xmpp_run_once(xmpp_ctx_t *ctx, unsigned long timeout)
             if (!(tsq->owner & XMPP_QUEUE_SM) && conn->sm_state->sm_enabled) {
                 tsq->sm_h = conn->sm_state->sm_sent_nr;
                 conn->sm_state->sm_sent_nr++;
-                strophe_debug_verbose(1, ctx, "xmpp", "SM_Q_MOVE: %p", tsq);
+                strophe_debug_verbose(1, ctx, "xmpp", "SM_Q_MOVE: %p, h=%lu",
+                                      tsq, tsq->sm_h);
                 add_queue_back(&conn->sm_state->sm_queue, tsq);
                 tsq = NULL;
             }
@@ -176,6 +175,7 @@ void xmpp_run_once(xmpp_ctx_t *ctx, unsigned long timeout)
             if (!sq)
                 conn->send_queue_tail = NULL;
         }
+        intf->flush(intf);
 
         /* tear down connection on error */
         if (conn->error) {
@@ -210,6 +210,7 @@ next_item:
     connitem = ctx->connlist;
     while (connitem) {
         conn = connitem->conn;
+        intf = &conn->intf;
 
         switch (conn->state) {
         case XMPP_STATE_CONNECTING:
@@ -244,7 +245,7 @@ next_item:
 
         /* Check if there is something in the SSL buffer. */
         if (conn->tls)
-            tls_read_bytes += tls_pending(conn->tls);
+            tls_read_bytes += tls_pending(intf);
 
         if (conn->state != XMPP_STATE_DISCONNECTED && conn->sock > max)
             max = conn->sock;
@@ -263,9 +264,9 @@ next_item:
 
     /* select errored */
     if (ret < 0) {
-        if (!sock_is_recoverable(sock_error()))
+        if (!sock_is_recoverable(NULL, sock_error(NULL)))
             strophe_error(ctx, "xmpp", "event watcher internal error %d",
-                          sock_error());
+                          sock_error(NULL));
         return;
     }
 
@@ -277,6 +278,7 @@ next_item:
     connitem = ctx->connlist;
     while (connitem) {
         conn = connitem->conn;
+        intf = &conn->intf;
 
         switch (conn->state) {
         case XMPP_STATE_CONNECTING:
@@ -304,14 +306,9 @@ next_item:
 
             break;
         case XMPP_STATE_CONNECTED:
-            if (FD_ISSET(conn->sock, &rfds) ||
-                (conn->tls && tls_pending(conn->tls))) {
-                if (conn->tls) {
-                    ret = tls_read(conn->tls, buf, STROPE_MESSAGE_BUFFER_SIZE);
-                } else {
-                    ret =
-                        sock_read(conn->sock, buf, STROPE_MESSAGE_BUFFER_SIZE);
-                }
+            if (FD_ISSET(conn->sock, &rfds) || intf->pending(intf)) {
+
+                ret = intf->read(intf, buf, STROPHE_MESSAGE_BUFFER_SIZE);
 
                 if (ret > 0) {
                     ret = parser_feed(conn->parser, buf, ret);
@@ -321,15 +318,13 @@ next_item:
                                         "parse error");
                     }
                 } else {
-                    if (conn->tls) {
-                        if (!tls_is_recoverable(tls_error(conn->tls))) {
-                            strophe_debug(ctx, "xmpp",
-                                          "Unrecoverable TLS error, %d.",
-                                          tls_error(conn->tls));
-                            conn->error = tls_error(conn->tls);
-                            conn_disconnect(conn);
-                        }
-                    } else {
+                    int err = intf->get_error(intf);
+                    if (!intf->error_is_recoverable(intf, err)) {
+                        strophe_debug(ctx, "xmpp", "Unrecoverable error: %d.",
+                                      err);
+                        conn->error = err;
+                        conn_disconnect(conn);
+                    } else if (!conn->tls) {
                         /* return of 0 means socket closed by server */
                         strophe_debug(ctx, "xmpp",
                                       "Socket closed by remote host.");
