@@ -1,3 +1,4 @@
+/* SPDX-License-Identifier: MIT OR GPL-3.0-only */
 /* stanza.c
 ** strophe XMPP client library -- XMPP stanza object and utilities
 **
@@ -6,7 +7,7 @@
 **  This software is provided AS-IS with no warranty, either express
 **  or implied.
 **
-**  This program is dual licensed under the MIT and GPLv3 licenses.
+**  This program is dual licensed under the MIT or GPLv3 licenses.
 */
 
 /** @file
@@ -41,15 +42,10 @@ xmpp_stanza_t *xmpp_stanza_new(xmpp_ctx_t *ctx)
 
     stanza = strophe_alloc(ctx, sizeof(xmpp_stanza_t));
     if (stanza != NULL) {
+        memset(stanza, 0, sizeof(xmpp_stanza_t));
         stanza->ref = 1;
         stanza->ctx = ctx;
         stanza->type = XMPP_STANZA_UNKNOWN;
-        stanza->prev = NULL;
-        stanza->next = NULL;
-        stanza->children = NULL;
-        stanza->parent = NULL;
-        stanza->data = NULL;
-        stanza->attributes = NULL;
     }
 
     return stanza;
@@ -182,6 +178,7 @@ int xmpp_stanza_release(xmpp_stanza_t *stanza)
         while (child) {
             tchild = child;
             child = child->next;
+            tchild->next = NULL;
             xmpp_stanza_release(tchild);
         }
 
@@ -1604,8 +1601,7 @@ xmpp_error_new(xmpp_ctx_t *ctx, xmpp_error_type_t type, const char *text)
     }
 
     xmpp_stanza_set_ns(error_type, XMPP_NS_STREAMS_IETF);
-    xmpp_stanza_add_child(error, error_type);
-    xmpp_stanza_release(error_type);
+    xmpp_stanza_add_child_ex(error, error_type, 0);
 
     if (text) {
         xmpp_stanza_t *error_text = xmpp_stanza_new(ctx);
@@ -1615,11 +1611,9 @@ xmpp_error_new(xmpp_ctx_t *ctx, xmpp_error_type_t type, const char *text)
         xmpp_stanza_set_ns(error_text, XMPP_NS_STREAMS_IETF);
 
         xmpp_stanza_set_text(content, text);
-        xmpp_stanza_add_child(error_text, content);
-        xmpp_stanza_release(content);
+        xmpp_stanza_add_child_ex(error_text, content, 0);
 
-        xmpp_stanza_add_child(error, error_text);
-        xmpp_stanza_release(error_text);
+        xmpp_stanza_add_child_ex(error, error_text, 0);
     }
 
     return error;
